//+------------------------------------------------------------------+
//|                                         ClassPriceHistogramA.mqh |
//|                                      Copyright vdv_2001 Software |
//|                                                 vdv_2001@mail.ru |
//+------------------------------------------------------------------+
#property copyright "Dmitry Voronkov"
#property link      "vdv_2001@mail.ru"

#include <Object.mqh>
#include <Arrays\List.mqh>
#include <ChartObjects\ChartObjectsLines.mqh>
#include <ChartObjects\ChartObjectsTxtControls.mqh>

//     / Amount of attempts of inquiry of the data
const int         AMOUNT_OF_ATTEMPTS=10;
//+------------------------------------------------------------------+
//|    CPriceHistogram                                          |
//+------------------------------------------------------------------+

class CPriceHistogram : public CObject
  {
private:
   //  
   double            high_day,low_day;

   //    POCs / Data for construction POCs
   CChartObjectTrend *POCLine;
   CChartObjectTrend *SecondTopPOCLine,*SecondBottomPOCLine;
   CChartObjectText *POCLable;
   bool              show_level;       //   / to show level
   bool              virgin;           //    / it is virgin level
   bool              show_second_poc;  //    POCs / to show secondary levels POCs
   datetime          shift_second_poc; //    POCs   / displacement of secondary levels POCs in seconds
   double            second_poc_top;   //     POCs / value of top level secondary POCs
   double            second_poc_bottom;//     POCs / value of the bottom level secondary POCs
   double            poc_value;        //   POCs / value of level POCs
   color             poc_color;        //   POCs / olour of level POCs
   datetime          poc_start_time;
   datetime          poc_end_time;
   //     / Data for Histogram construction
//   CList             ListHistogramInner;//     Inner / for storage of lines of the histogram Inner
//   CList             ListHistogramOuter;//     Outer / for storage of lines of the histogram Outer
   CList             ListHistogram;
   bool              show_histogram;   //    /  to show the histogram
   color             inner_color;      //    / inner colour of the histogram
   color             outer_color;      //    / outer colour of the histogram
   uint              range_percent;    //   / range percent
   datetime          time_start;       //     / time to start construction
   datetime          time_end;         //    / time of end of construction
public:
   //   / Class constructor
                     CPriceHistogram();
   //   / Class destructor
                    ~CPriceHistogram(){Delete();}
   //   / Class initialization
   bool              Init(datetime time_open,datetime time_close,bool showhistogram);
   //    / To show value of level
   void              ShowLevel(bool show){show_level=show; RefreshPOCs();}
   bool              ShowLevel(){return(show_level);}
   //   / To show the histogram
   void              ShowHistogram(bool show);
   bool              ShowHistogram(){return(show_histogram);}
   //    POCs / To show Secondary levels POCs
   void              ShowSecondaryPOCs(bool show,datetime seconds){show_second_poc=show;shift_second_poc=seconds;RefreshPOCs();}
   bool              ShowSecondaryPOCs(){return(show_second_poc);}
   //    POCs / To establish colour of levels POCs
   void              ColorPOCs(color col){poc_color=col; RefreshPOCs();}
   color             ColorPOCs(){return(poc_color);}
   //     / To establish internal colour of the histogram
   void              ColorInner(color col){inner_color=col; calculationHistogram();}
   color             ColorInner(){return(inner_color);}
   //     / To establish outer colour of the histogram
   void              ColorOuter(color col){outer_color=col; calculationHistogram();}
   color             ColorOuter(){return(outer_color);}
   //    / To establish range percent
   void              RangePercent(uint percent){range_percent=percent; calculationHistogram();}
   uint              RangePercent(){return(range_percent);}
   //     POCs / Returns value of virginity of level POCs
   bool              VirginPOCs(){return(virgin);}
   //      / Returns starting time of construction of the histogram
   datetime          GetStartDateTime(){return(time_start);}
   bool              RefreshPOCs();
private:
   bool              calculationHistogram();
   bool              calculationPOCs();
   int               refreshHistogram(color inner,color outer);
   //   / Class removal
   void              Delete();
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CPriceHistogram::CPriceHistogram()
  {
   POCLine=NULL;
   SecondTopPOCLine=NULL;
   SecondBottomPOCLine=NULL;
   POCLable=NULL;
   show_level=true;
   show_second_poc=false;
   show_histogram=false;
   shift_second_poc=0;
   virgin=false;
   poc_color=Orange;
   inner_color=Indigo;
   outer_color=Magenta;
   range_percent=70;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CPriceHistogram::Delete()
  {
   if(POCLine!=NULL) delete POCLine;
   if(POCLable!=NULL) delete POCLable;
   if(SecondTopPOCLine!=NULL) delete SecondTopPOCLine;
   if(SecondBottomPOCLine!=NULL) delete SecondBottomPOCLine;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CPriceHistogram::ShowHistogram(bool show)
  {
   if(show_histogram==show) return;
   show_histogram=show;
   if(show_histogram)
      calculationHistogram();
   else
     {
//      ListHistogramInner.Clear();
//      ListHistogramOuter.Clear();
      ListHistogram.Clear();
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CPriceHistogram::Init(datetime time_open,datetime time_close,bool showhistogram)
  {
   time_start=time_open;
   time_end=time_close;
   calculationPOCs();
   RefreshPOCs();
   ShowHistogram(showhistogram);
   return(true);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CPriceHistogram::calculationHistogram()
  {
   int rates_total,rates_high,rates_time,index;
   double iHigh[],iLow[];
   datetime iTime[];
//      time_start  time_end / We obtain the data from time_start to time_end
   int err=0;
   do
     {
      rates_time=CopyTime(NULL,PERIOD_M1,time_start,time_end,iTime);
      rates_high=CopyHigh(NULL,PERIOD_M1,time_start,time_end,iHigh);
      rates_total=CopyLow(NULL,PERIOD_M1,time_start,time_end,iLow);
      err++;
     }
   while((rates_time<=0 || (rates_total!=rates_high && rates_total!=rates_time)) && err<AMOUNT_OF_ATTEMPTS);
   if(err>=AMOUNT_OF_ATTEMPTS)
     {
      return(false);
     }
   high_day=iHigh[ArrayMaximum(iHigh,WHOLE_ARRAY,rates_total)];
   low_day=iLow[ArrayMinimum(iLow,WHOLE_ARRAY,rates_total)];
   double factor=_Point;
   int count=int((high_day-low_day)/factor)+2;
//       
   int ThicknessOfLevel[];    //      / we create an array for count of tics
   ArrayResize(ThicknessOfLevel,count);
   ArrayInitialize(ThicknessOfLevel,0);
   for(int i=0;i<rates_total;i++)
     {
      double C=iLow[i];
      while(C<iHigh[i])
        {
         int Index=int((C-low_day)/factor);
         ThicknessOfLevel[Index]++;
         C+=factor;
        }
     }
   int MaxLevel=ArrayMaximum(ThicknessOfLevel,WHOLE_ARRAY,count);
   int Range=ThicknessOfLevel[MaxLevel]-ThicknessOfLevel[MaxLevel]*range_percent/100;
   int DownLine=0;
   int UpLine=0;
   for(int i=0;i<count;i++)
     {
      if(ThicknessOfLevel[i]>=Range)
        {
         DownLine=i;
         break;
        }
     }
   for(int i=count-1;i>0;i--)
     {
      if(ThicknessOfLevel[i]>=Range)
        {
         UpLine=i;
         break;
        }
     }
   if(DownLine==0)
      DownLine=MaxLevel;
   if(UpLine==0)
      UpLine=MaxLevel;
   datetime Delta=(iTime[rates_total-1]-iTime[0]-PeriodSeconds(PERIOD_H1))/ThicknessOfLevel[MaxLevel];
//   ListHistogramInner.Clear();
//   ListHistogramOuter.Clear();
   ListHistogram.Clear();
   for(int i=0;i<count;i++)
     {
      string name=TimeToString(time_start)+" "+IntegerToString(i);
      double StartY= low_day+factor*i;
      datetime EndX= iTime[0]+(ThicknessOfLevel[i])*Delta;

      CChartObjectTrend *obj=new CChartObjectTrend();
      obj.Create(0,name,0, iTime[0],StartY,EndX,StartY);
      obj.Background(true);
      if(i>=DownLine && i<=UpLine)
        {
         obj.Color(inner_color);
//         ListHistogramInner.Add(obj);
        }
      else
        {
         obj.Color(outer_color);
//         ListHistogramOuter.Add(obj);
        }
        ListHistogram.Add(obj);
     }

   return(true);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CPriceHistogram::refreshHistogram(color inner,color outer)
  {
   int result;
   if(!show_histogram) return(-1);

   return(result);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CPriceHistogram::calculationPOCs()
  {
   int rates_total,rates_high,rates_time,index;
   double iHigh[],iLow[];
   datetime iTime[];
//      time_start  time_end / We obtain the data from time_start to time_end
   int err=0;
   do
     {
      rates_time=CopyTime(NULL,PERIOD_M1,time_start,time_end,iTime);
      rates_high=CopyHigh(NULL,PERIOD_M1,time_start,time_end,iHigh);
      rates_total=CopyLow(NULL,PERIOD_M1,time_start,time_end,iLow);
      err++;
     }
   while((rates_time<=0 || (rates_total!=rates_high && rates_total!=rates_time)) && err<AMOUNT_OF_ATTEMPTS);
   if(err>=AMOUNT_OF_ATTEMPTS)
     {
      return(false);
     }
   poc_start_time=iTime[0];
   high_day=iHigh[ArrayMaximum(iHigh,WHOLE_ARRAY,rates_total)];
   low_day=iLow[ArrayMinimum(iLow,WHOLE_ARRAY,rates_total)];
   int count=int((high_day-low_day)/_Point)+1;
//        / Count of duration of a finding of the price at each level
   int ThicknessOfLevel[];    //      / we create an array for count of tics
   ArrayResize(ThicknessOfLevel,count);
   ArrayInitialize(ThicknessOfLevel,0);
   for(int i=0;i<rates_total;i++)
     {
      double C=iLow[i];
      while(C<iHigh[i])
        {
         int Index=int((C-low_day)/_Point);
         ThicknessOfLevel[Index]++;
         C+=_Point;
        }
     }
   int MaxLevel=ArrayMaximum(ThicknessOfLevel,WHOLE_ARRAY,count);
   poc_value=low_day+_Point*MaxLevel;
//   POCs / We find secondary POCs
   int range_min=ThicknessOfLevel[MaxLevel]-ThicknessOfLevel[MaxLevel]*range_percent/100;
   second_poc_bottom=0;
   second_poc_top=0;
   for(int i=0;i<count;i++)
     {
      if(ThicknessOfLevel[i]>=range_min)
        {
         second_poc_bottom=i;
         break;
        }
     }
   for(int i=count-1;i>0;i--)
     {
      if(ThicknessOfLevel[i]>=range_min)
        {
         second_poc_top=i;
         break;
        }
     }
   if(second_poc_bottom==0)
      second_poc_bottom=MaxLevel;
   if(second_poc_top==0)
      second_poc_top=MaxLevel;
   second_poc_top=low_day+_Point*second_poc_top;
   second_poc_bottom=low_day+_Point*second_poc_bottom;
   datetime current=StringToTime(TimeToString(TimeCurrent(),TIME_DATE));
//      POC   / We check the given level is virgin POC or not
   err=0;
   do
     {
      rates_time=CopyTime(NULL,PERIOD_M1,time_end,current,iTime);
      rates_high=CopyHigh(NULL,PERIOD_M1,time_end,current,iHigh);
      rates_total=CopyLow(NULL,PERIOD_M1,time_end,current,iLow);
      err++;
     }
   while((rates_time<=0 || (rates_total!=rates_high && rates_total!=rates_time)) && err<AMOUNT_OF_ATTEMPTS);
   if(rates_time==0)
     {
      virgin=true;
     }
   else
     {
      for(index=0;index<rates_total;index++)
         if(poc_value<iHigh[index] && poc_value>iLow[index]) break;

      if(index<rates_total)
         poc_end_time=iTime[index];
      else
         virgin=true;
     }
   if(POCLine==NULL)
     {
      POCLine=new CChartObjectTrend();
      POCLine.Create(0,TimeToString(time_start)+" POC ",0,poc_start_time,poc_value,0,0);
      POCLine.Color(poc_color);
     }
   RefreshPOCs();
   return(true);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CPriceHistogram::RefreshPOCs()
  {
   if(POCLine==NULL) return(false);
   if(virgin)
     {
      MqlTick last_tick;
      datetime cur_time;
      if(SymbolInfoTick(Symbol(),last_tick))
         cur_time=last_tick.time;
      else
         cur_time=TimeCurrent();
      POCLine.SetPoint(1,cur_time,poc_value);
      if(show_level)
        {
         if(POCLable==NULL)
           {
            POCLable=new CChartObjectText();
            POCLable.Create(0,POCLine.Name()+"L",0,0,0);
            POCLable.Anchor(ANCHOR_LEFT);
            POCLable.FontSize(7);
            POCLable.Font("Tahoma");
            POCLable.Color(poc_color);
           }
         POCLable.Description(DoubleToString(poc_value,_Digits));
         POCLable.SetPoint(0,cur_time+PeriodSeconds(PERIOD_CURRENT),poc_value);
        }
      else
         if(POCLable!=NULL) delete POCLable;
     }
   else
      POCLine.SetPoint(1,poc_end_time,poc_value);
   if(show_second_poc)
     {
      if(SecondTopPOCLine==NULL)
        {
         SecondTopPOCLine=new CChartObjectTrend();
         SecondTopPOCLine.Create(0,TimeToString(time_start)+" SecondTopPOC ",0,poc_start_time, second_poc_top,0,0);
         SecondTopPOCLine.Color(poc_color);
         SecondTopPOCLine.Style(STYLE_DOT);
        }
      SecondTopPOCLine.SetPoint(1,time_end+shift_second_poc,second_poc_top);
      if(SecondBottomPOCLine==NULL)
        {
         SecondBottomPOCLine=new CChartObjectTrend();
         SecondBottomPOCLine.Create(0,TimeToString(time_start)+" SecondBottomPOC ",0,poc_start_time, second_poc_bottom,0,0);
         SecondBottomPOCLine.Color(poc_color);
         SecondBottomPOCLine.Style(STYLE_DOT);
        }
      SecondBottomPOCLine.SetPoint(1,time_end+shift_second_poc,second_poc_bottom);
     }
   else
     {
      if(SecondTopPOCLine!=NULL) {delete SecondTopPOCLine; SecondTopPOCLine=NULL;};
      if(SecondBottomPOCLine!=NULL) {delete SecondBottomPOCLine;SecondBottomPOCLine=NULL;}
     }
   return(true);
  }
//+------------------------------------------------------------------+
